module DataMapper
  module Model
    module Relationship
      def blob(name)
        add_relation_property(name)
        add_model_reader(name)
        add_model_writer(name)
        
        add_before_save_method(name)
        add_before_destroy_method(name)
        
        add_filter_methods(name)
      
      end
      
      private
      
      def add_model_reader(name)
        class_eval <<-RUBY, __FILE__, __LINE__ + 1
          chainable do
            def #{name}
              if self.#{name}_id && !@#{name}
                @#{name} = AppEngineUtils::Datastore::Blob.new.read(self.#{name}_id)
              end
              @#{name}
              
            end
          end
        RUBY
      end
      
      #TODO: add model_writer
      def add_model_writer(name)
        class_eval <<-RUBY, __FILE__, __LINE__ + 1
          chainable do
            def #{name}=(value)
              @#{name}=value
            end
          end
        RUBY
      end
      
      def add_relation_property(name)
        class_eval <<-RUBY, __FILE__, __LINE__ + 1
          property :#{name}_id, String
        RUBY
      end
      
      def add_filter_methods(name)
        class_eval <<-RUBY, __FILE__, __LINE__ + 1
          before :save, :before_save_#{name}
          before :destroy, :before_destroy_#{name}
        RUBY
      end
      
      def add_before_save_method(name)
        class_eval <<-RUBY, __FILE__, __LINE__ + 1
          def before_save_#{name}
            if self.#{name}_id
              blob = AppEngineUtils::Datastore::Blob.new
              blob.delete(self.#{name}_id)
            end
            if #{name}
              blob = AppEngineUtils::Datastore::Blob.new
              key = blob.create(#{name})
              self.#{name}_id= key
            end
          end
        RUBY
      end
      
      def add_before_destroy_method(name)
        class_eval <<-RUBY, __FILE__, __LINE__ + 1
          def before_destroy_#{name}
            if self.#{name}_id
              blob = AppEngineUtils::Datastore::Blob.new
              blob.delete(self.#{name}_id)
            end  
          end
        RUBY
      end
      
    end  
  end
end